@@ -14,7 +14,7 @@ use namespace::clean -except => [ qw(meta
                                      merged_section_data
                                      merged_section_data_names) ];
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 with qw(Hailo::Role::Generic
         Hailo::Role::Storage
@@ -220,10 +220,6 @@ sub _engage {
 sub start_training {
     my ($self) = @_;
     $self->_engage() if !$self->_engaged;
-    my $st = $self->dbd eq 'mysql'
-        ? "ALTER TABLE next_token DROP INDEX next_token_token_id;"
-        : "DROP INDEX next_token_token_id;";
-    $self->dbh->do($st);
     $self->start_learning();
     return;
 }
@@ -231,7 +227,6 @@ sub start_training {
 sub stop_training {
     my ($self) = @_;
     $self->stop_learning();
-    $self->dbh->do('CREATE INDEX next_token_token_id ON next_token (token_id);');
     return;
 }
 
@@ -285,20 +280,21 @@ sub _get_create_db_sql {
     return ($sql =~ /\s*(.*?);/gs);
 }
 
-# return the number of tokens we know about
-sub token_total {
+# return some statistics
+sub totals {
     my ($self) = @_;
     $self->_engage() if !$self->_engaged;
-    $self->sth->{token_total}->execute();
-    return $self->sth->{token_total}->fetchrow_array - 1;
-}
 
-# return the number of expressions we know about
-sub expr_total {
-    my ($self) = @_;
-    $self->_engage() if !$self->_engaged;
+    $self->sth->{token_total}->execute();
+    my $token = $self->sth->{token_total}->fetchrow_array - 1;
     $self->sth->{expr_total}->execute();
-    return $self->sth->{expr_total}->fetchrow_array // 0;
+    my $expr = $self->sth->{expr_total}->fetchrow_array // 0;
+    $self->sth->{prev_total}->execute();
+    my $prev = $self->sth->{prev_total}->fetchrow_array // 0;
+    $self->sth->{next_total}->execute();
+    my $next = $self->sth->{next_total}->fetchrow_array // 0;
+
+    return $token, $expr, $prev, $next;
 }
 
 ## no critic (Subroutines::ProhibitExcessComplexity)
@@ -309,10 +305,16 @@ sub make_reply {
     my $order = $self->order;
 
     # we will favor these tokens when making the reply
-    my @key_ids;
+    my @key_tokens = @$tokens;
 
-    my %token_cache;
-    for my $token_info (@$tokens) {
+    # shuffle the tokens and discard half of them
+    @key_tokens = do {
+        my $i = 0;
+        grep { $i++ % 2 == 0 } shuffle(@key_tokens);
+    };
+
+    my (@key_ids, %token_cache);
+    for my $token_info (@key_tokens) {
         my $text = $token_info->[1];
         my $info = $self->_token_similar($text);
         next if !defined $info;
@@ -323,14 +325,8 @@ sub make_reply {
         $token_cache{$id} = [$spacing, $text];
     }
 
-    # toss all but a third of the tokens away
-    @key_ids = do {
-        my $i = 0;
-        grep { $i++ % 2 == 0 } shuffle(@key_ids);
-    };
-
     # sort the rest by rareness
-    @key_ids = $self->_sort_rare_tokens(\@key_ids);
+    @key_ids = $self->_find_rare_tokens(\@key_ids, 2);
 
     # get the middle expression
     my $seed_token_id = shift @key_ids;
@@ -404,7 +400,11 @@ sub learn_tokens {
     for my $i (0 .. @$tokens - $order) {
         my @expr = map { $token_cache{ join('', @{ $tokens->[$_] }) } } $i .. $i+$order-1;
         my $expr_id = $self->_expr_id(\@expr);
-        $expr_id = $self->_add_expr(\@expr) if !defined $expr_id;
+
+        if (!defined $expr_id) {
+            $expr_id = $self->_add_expr(\@expr);
+            $self->sth->{inc_token_count}->execute($_) for uniq(@expr);
+        }
 
         # add link to next token for this expression, if any
         if ($i < @$tokens - $order) {
@@ -428,18 +428,22 @@ sub learn_tokens {
 }
 
 # sort token ids based on how rare they are
-sub _sort_rare_tokens {
-    my ($self, $token_ids) = @_;
+sub _find_rare_tokens {
+    my ($self, $token_ids, $min) = @_;
     return if !@$token_ids;
 
-    my %rare;
+    my %links;
     for my $id (@$token_ids) {
-        next if exists $rare{$id};
+        next if exists $links{$id};
         $self->sth->{token_count}->execute($id);
-        $rare{$id} = $self->sth->{token_count}->fetchrow_array // 0;
+        $links{$id} = $self->sth->{token_count}->fetchrow_array;
     }
 
-    my @ids = sort { $rare{$a} <=> $rare{$b} } @$token_ids;
+    # remove tokens which are too rare
+    my @ids = grep { $links{$_} >= $min } @$token_ids;
+
+    @ids = sort { $links{$a} <=> $links{$b} } @ids;
+
     return @ids;
 }
 
@@ -450,8 +454,7 @@ sub _inc_link {
     my $count = $self->sth->{"${type}_count"}->fetchrow_array;
 
     if (defined $count) {
-        my $new_count = $count++;
-        $self->sth->{"${type}_inc"}->execute($new_count, $expr_id, $token_id);
+        $self->sth->{"${type}_inc"}->execute($expr_id, $token_id);
     }
     else {
         $self->sth->{"${type}_add"}->execute($expr_id, $token_id);
@@ -615,7 +618,8 @@ CREATE TABLE token (
             [% CASE DEFAULT %]INTEGER PRIMARY KEY AUTOINCREMENT,
          [% END %]
     spacing INTEGER NOT NULL,
-    text [% IF dbd == 'mysql' %] VARCHAR(255) [% ELSE %] TEXT [% END %] NOT NULL
+    text [% IF dbd == 'mysql' %] VARCHAR(255) [% ELSE %] TEXT [% END %] NOT NULL,
+    count INTEGER NOT NULL
 );
 __[ table_expr ]__
 CREATE TABLE expr (
@@ -658,7 +662,6 @@ CREATE INDEX token_text on token (text);
 CREATE INDEX expr_token_ids on expr ([% columns %]);
 CREATE INDEX next_token_expr_id ON next_token (expr_id);
 CREATE INDEX prev_token_expr_id ON prev_token (expr_id);
-CREATE INDEX next_token_token_id ON next_token (token_id);
 __[ static_query_get_order ]__
 SELECT text FROM info WHERE attribute = 'markov_order';
 __[ static_query_set_order ]__
@@ -667,6 +670,10 @@ __[ static_query_token_total ]__
 SELECT COUNT(id) FROM token;
 __[ static_query_expr_total ]__
 SELECT COUNT(id) FROM expr;
+__[ static_query_prev_total ]__
+SELECT COUNT(id) FROM prev_token;
+__[ static_query_next_total ]__
+SELECT COUNT(id) FROM next_token;
 __[ static_query_random_expr ]__
 SELECT * from expr
 [% SWITCH dbd %]
@@ -686,7 +693,10 @@ SELECT id, spacing FROM token WHERE text = ?
     [% CASE DEFAULT  %]ORDER BY RANDOM() LIMIT 1;
 [% END %]
 __[ static_query_add_token ]__
-INSERT INTO token (spacing, text) VALUES (?, ?)[% IF dbd == 'Pg' %] RETURNING id[% END %];
+INSERT INTO token (spacing, text, count) VALUES (?, ?, 0)
+[% IF dbd == 'Pg' %] RETURNING id[% END %];
+__[ static_query_inc_token_count ]__
+UPDATE token SET count = count + 1 WHERE id = ?;
 __[ static_query_last_expr_rowid ]_
 SELECT id FROM expr ORDER BY id DESC LIMIT 1;
 __[ static_query_last_token_rowid ]__
@@ -694,15 +704,16 @@ SELECT id FROM token ORDER BY id DESC LIMIT 1;
 __[ static_query_(next_token|prev_token)_count ]__
 SELECT count FROM [% table %] WHERE expr_id = ? AND token_id = ?;
 __[ static_query_(next_token|prev_token)_inc ]__
-UPDATE [% table %] SET count = ? WHERE expr_id = ? AND token_id = ?
+UPDATE [% table %] SET count = count + 1 WHERE expr_id = ? AND token_id = ?
 __[ static_query_(next_token|prev_token)_add ]__
 INSERT INTO [% table %] (expr_id, token_id, count) VALUES (?, ?, 1);
 __[ static_query_(next_token|prev_token)_get ]__
 SELECT token_id, count FROM [% table %] WHERE expr_id = ?;
 __[ static_query_token_count ]__
-SELECT COUNT(count) FROM next_token WHERE token_id = ?;
+SELECT count FROM token WHERE id = ?;
 __[ dynamic_query_(add_expr) ]__
-INSERT INTO expr ([% columns %]) VALUES ([% ids %])[% IF dbd == 'Pg' %] RETURNING id[% END %];
+INSERT INTO expr ([% columns %]) VALUES ([% ids %])
+[% IF dbd == 'Pg' %] RETURNING id[% END %];
 __[ dynamic_query_expr_by_token(NUM)_id ]__
 SELECT * FROM expr WHERE [% column %] = ?
 [% SWITCH dbd %]
